/*
 * Last updated: 2011/12/02
 * ~Philip Peng
 * 
 * This file is part of BomberNome.
 * 
 * BomberNome is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * BomberNome is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with BomberNome.	If not, see <http://www.gnu.org/licenses/>.
 */

#include "bombernome.h"

// Variables
// Game variables
unsigned int game_counter;
player_t player_1, player_2, player_3, player_4;
char game_grid[GRID_WIDTH][GRID_HEIGHT];

// Main
void main() {

	// Terminal
	TERMIO_Init();
#ifdef DEBUG
	printf("=========================\r\n");
	printf("  Welcome to BomberNome\r\n");
	printf("=========================\r\n");
#endif
	
	// Setup everything
	setup_display();
	setup_input();
	setup_game();
	
	// Start things up
	game_counter = 0;
	update_game();
	setup_timer();
	EnableInterrupts;
	
	// Infinite loop
	for(;;) {
		// TODO - for testing
		if (game_counter % 8 == 0) { //Every ~2s
			simulate_input();
			//update_game();
		}
		update_display();
	}
}

// For testing without real input
void simulate_input() {
  static unsigned short i;
  i++;                  
	if (i == 18) i = 0;
	if (i < 3) {
		player_1.action = ACTION_RIGHT;
		player_2.action = ACTION_DOWN;
		player_3.action = ACTION_UP;
		player_4.action = ACTION_LEFT;
	} else if (i < 9) {
		player_1.action = ACTION_NONE;
		player_2.action = ACTION_NONE;
		player_3.action = ACTION_NONE;
		player_4.action = ACTION_NONE;
	} else if (i < 12) {
		player_1.action = ACTION_LEFT;
		player_2.action = ACTION_UP;
		player_3.action = ACTION_DOWN;
		player_4.action = ACTION_RIGHT;
	} else if (i < 18) {
		player_1.action = ACTION_NONE;
		player_2.action = ACTION_NONE;
		player_3.action = ACTION_NONE;
		player_4.action = ACTION_NONE;
	}  
}

// Called by main, set up game update timer
void setup_timer() {
  // Game clock
  // MC9S12C128 has a 2MHz bus clock
  // 16-bit = 2^16 = 65536 cycles
  // When PR2 = 0, PR1 = 1, PR0 = 1, timer = bus / 8
  // [1 / (2MHz / 8)] * 65536 cycles = 0.2621s
  // TSCR2 = [TOI][---][---][---][TCRE][PR2][PR1][PR0]
  // TIO = Timer Overflow Interrupt Enable (want on for interrupt)
  // TCRE = Timer Counter Reset Enable (want off since not comparing)
	// 0b10000011 = 0x83
	TSCR2 = 0x83;				// Enable timer 2 for overflow, prescale by 8

}

// Standard Timer Overflow (clock)
void interrupt 16 timer_overflow(){
	TFLG2 &= 0x80;	// Clear TOF flag
	update_game();	// Update game
}

// Real Time Interrupt (input)
//void interrupt 7 input_interrupt() {
	//CRGFLG = 0x80;			 // Clear interrupt flag
	//update_actions();	// Update player actions
//}

// For debugging, print out the grid
#ifdef DEBUG
void print_game_grid() {
	byte x, y;
	printf("=========================\r\n");
	printf("game_grid at game_counter = %d\r\n", game_counter);
	for (y = 0; y < GRID_HEIGHT; y++) {
		for (x = 0; x < GRID_WIDTH; x++) {
			printf("%c ", game_grid[x][y]);
		}
		printf("\r\n");
	}
	printf("=========================\r\n");
}
#endif

